package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.Entity;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.StrConstant;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.flow.entity.FlowAudit;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.lis.config.LisDb;
import com.github.jfcloud.gene.sample.entity.*;
import com.github.jfcloud.gene.sample.enums.AgeUnitEnum;
import com.github.jfcloud.gene.sample.enums.PathogenSampleTypeEnum;
import com.github.jfcloud.gene.sample.enums.ScientificSampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.InspectionCommissionMapper;
import com.github.jfcloud.gene.sample.mapper.SampleAnimalMapper;
import com.github.jfcloud.gene.sample.mapper.SampleCheckOrderMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.*;
import com.github.jfcloud.gene.sample.vo.*;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.gene.util.WordUtil;
import com.github.jfcloud.rich.config.WordConfig;
import com.github.jfcloud.rich.vo.HtmlParseVo;
import com.github.jfcloud.rich.word.WordNodeFactory;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
@Service
@RequiredArgsConstructor
public class SampleAnimalServiceImpl extends ServiceImpl<SampleAnimalMapper, SampleAnimal> implements SampleAnimalService, SampleEditService {

    private final SampleInfoMapper sampleInfoMapper;
    private final InspectionCommissionMapper inspectionCommissionMapper;
    private final SampleCheckOrderMapper checkOrderMapper;
    private final CommissionDetailService commissionDetailService;
    private final SampleCheckItemService sampleCheckItemService;
    private final SampleCheckItemSerumService sampleCheckItemSerumService;
    private final WordDataService wordDataService;
    private final FileInfoService fileInfoService;
    private final UserIdNameCache userIdNameCache;
    private final FlowAuditService flowAuditService;
    private final CommonSampleEditServiceImpl commonSampleEditService;
    private final LisDb lisDb;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SampleAnimal> sampleAnimals = list(new LambdaQueryWrapper<>(SampleAnimal.class)
                .select(SampleAnimal::getId)
                .eq(SampleAnimal::getSampleId, sampleId));
        if (sampleAnimals.isEmpty()) {
            return;
        }

        Set<Long> sampleAnimalIdSet = sampleAnimals.stream().map(SampleAnimal::getId).collect(Collectors.toSet());
        removeBatchByIds(sampleAnimalIdSet);

        //删除委托单
        List<InspectionCommission> inspectionCommissions = inspectionCommissionMapper.selectList(new LambdaQueryWrapper<>(InspectionCommission.class)
                .select(InspectionCommission::getId)
                .in(InspectionCommission::getSampleAnimalId, sampleAnimalIdSet));
        Set<Long> commissionIdSet = inspectionCommissions.stream().map(InspectionCommission::getId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(commissionIdSet)) {
            inspectionCommissionMapper.deleteBatchIds(commissionIdSet);
            commissionDetailService.remove(new LambdaQueryWrapper<>(CommissionDetail.class).in(CommissionDetail::getInspectionCommissionId, commissionIdSet));
        }

        //删除送检申请单
        List<SampleCheckOrder> orders = checkOrderMapper.selectList(new LambdaQueryWrapper<>(SampleCheckOrder.class)
                .select(SampleCheckOrder::getId)
                .in(SampleCheckOrder::getSampleAnimalId, sampleAnimalIdSet));
        Set<Long> orderIdSet = orders.stream().map(SampleCheckOrder::getId).collect(Collectors.toSet());
        if (CollUtil.isNotEmpty(orderIdSet)) {
            checkOrderMapper.deleteBatchIds(orderIdSet);
            sampleCheckItemService.remove(new LambdaQueryWrapper<>(SampleCheckItem.class).in(SampleCheckItem::getCheckOrderId, orderIdSet));
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        Assert.notNull(vo.getAnimal(), "动物样本送检不能为空");

        //先删除之前的记录
        removeBySampleId(sampleId);

        //保存动物样本送检信息
        SampleAnimal sampleAnimal = BeanUtil.copyProperties(vo.getAnimal(), SampleAnimal.class);
        sampleAnimal.setId(CustomIdGenerator.nextId());
        sampleAnimal.setSampleId(sampleId);
        sampleAnimal.insert();

        //保存委托列表，通过JSON序列化转为列表
        List<String> paTypes = PathogenSampleTypeEnum.resolveList(vo.getAnimal().getPathogenSampleType());
        CommissionObjVo commissionObj = vo.getAnimal().getCommission();
        JSONObject commissionJson = JSON.parseObject(JSON.toJSONString(commissionObj, SerializerFeature.WriteMapNullValue));
        for (String fieldName : paTypes) {
            JSONObject fieldJson = commissionJson.getJSONObject(fieldName);
            if (fieldJson == null) {
                continue;
            }
            InspectionCommission commission = JSON.parseObject(fieldJson.toJSONString(), InspectionCommission.class);
            commission.setDetectType(fieldName);
            commission.setId(CustomIdGenerator.nextId());
            commission.setSampleAnimalId(sampleAnimal.getId());
            commission.insert();

            //保存样本类型
            JSONArray array = fieldJson.getJSONArray("detailList");
            if (array.isEmpty()) {
                log.warn("{} 委托样本类型为空", fieldName);
                continue;
            }

            List<CommissionDetail> commissionDetails = JSON.parseArray(array.toJSONString(), CommissionDetail.class)
                    .stream()
                    .filter(item -> StrUtil.isNotBlank(item.getSampleType()))
                    .peek(item -> item.setInspectionCommissionId(commission.getId()))
                    .collect(Collectors.toList());
            commissionDetailService.saveBatch(commissionDetails);
        }

        //保存送检申请单
        List<String> scTypes = ScientificSampleTypeEnum.resolveList(vo.getAnimal().getScientificSampleType());
        CheckOrderObjVo orderObj = vo.getAnimal().getOrder();
        JSONObject orderJson = JSON.parseObject(JSON.toJSONString(orderObj, SerializerFeature.WriteMapNullValue));
        for (String fieldName : scTypes) {
            JSONObject fieldJson = orderJson.getJSONObject(fieldName);
            if (fieldJson == null) {
                continue;
            }
            SampleCheckOrder order = JSON.parseObject(fieldJson.toJSONString(), SampleCheckOrder.class);
            order.setApplyType(fieldName);
            order.setId(CustomIdGenerator.nextId());
            order.setSampleAnimalId(sampleAnimal.getId());
            order.insert();

            //保存送检申请单条目
            JSONArray array = fieldJson.getJSONArray("items");
            if (array.isEmpty()) {
                log.warn("type={} orderCode={} 送检申请单条目为空", fieldName, order.getOrderCode());
                continue;
            }

            if (ScientificSampleTypeEnum.serum.name().equals(fieldName)) {
                List<SampleCheckItemSerum> sampleCheckItemSerums = JSON.parseArray(array.toJSONString(), SampleCheckItemSerum.class);
                sampleCheckItemSerums.forEach(item -> item.setCheckOrderId(order.getId()));
                sampleCheckItemSerumService.saveBatch(sampleCheckItemSerums);
                continue;
            }

            List<SampleCheckItem> sampleCheckItems = JSON.parseArray(array.toJSONString(), SampleCheckItem.class);
            sampleCheckItems.forEach(item -> item.setCheckOrderId(order.getId()));
            sampleCheckItemService.saveBatch(sampleCheckItems);
        }

        log.info("保存动物样本送检信息成功");
        generate(sampleId);
    }

    @SneakyThrows
    @Override
    public void generate(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        SampleAnimalVo sampleAnimal = getDetail(sampleId);

        //委托列表扩展字段
        sampleAnimal.getCommission().getPig().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getPig().getExt()));
        sampleAnimal.getCommission().getRodent().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getRodent().getExt()));
        sampleAnimal.getCommission().getMonkey().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getMonkey().getExt()));
        sampleAnimal.getCommission().getDog().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getDog().getExt()));
        sampleAnimal.getCommission().getCat().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getCat().getExt()));
        sampleAnimal.getCommission().getGoat().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getGoat().getExt()));
        sampleAnimal.getCommission().getMaterial().setExtObj(JSONObject.parseObject(sampleAnimal.getCommission().getMaterial().getExt()));

        //血清样本送检
        if (StrUtil.isNotBlank(sampleAnimal.getScientificSampleType()) &&
                sampleAnimal.getScientificSampleType().contains(ScientificSampleTypeEnum.serum.getDescription())) {
            //获取检测指标
            String sql = "select id, name from dbo.HT_DICT_ITEM where status=1 and type=0";
            List<Entity> entities = lisDb.query(sql);
            Map<String, String> dictItemMap = entities.stream()
                    .collect(Collectors.toMap(e -> e.getStr("id"), e -> e.getStr("name")));

            sampleAnimal.getOrder().getSerum().getItems().forEach(item -> {
                //加急
                item.setQuick(WhetherEnum.getLabel(item.getQuick()));
                //年龄
                item.setAge(item.getAge() + AgeUnitEnum.getLabel(item.getAgeUnit()));
                //检测指标
                if (StrUtil.isNotBlank(item.getTarget())) {
                    String target = Stream.of(item.getTarget().split(","))
                            .map(dictItemMap::get)
                            .collect(Collectors.joining("、"));
                    item.setTarget(target);
                }
            });
        }

        //文件名称：项目名称-申请人-申请日期-随机四位.docx
        String applyDate = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("动物样本检测送检申请表-%s-%s-%s-%s.docx",
                StrUtil.nullToEmpty(sampleInfo.getProjectName()), StrUtil.nullToEmpty(sampleInfo.getApplicantName()), applyDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 项目名称：{} 文件名称：{}", sampleInfo.getProjectName(), fileName);

        JSONObject dataMap = new JSONObject();
        wordDataService.fillDocCreator(dataMap);
        JSONObject sampleInfoObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleInfo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleInfo", sampleInfoObj);
        JSONObject sampleAnimalObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleAnimal, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleAnimal", sampleAnimalObj);

        //检测人员
        if (StrUtil.isNotEmpty(sampleInfo.getInspector())) {
            String inspectorName = JSON.parseArray(sampleInfo.getInspector())
                    .stream()
                    .map(item -> {
                        JSONObject obj = (JSONObject) item;
                        return obj.getString("name");
                    })
                    .collect(Collectors.joining("、"));
            sampleInfoObj.put("inspectorName", inspectorName);
        }

        //富文本图片
        JSONArray images = new JSONArray();
        dataMap.put("images", images);

        //富文本
        WordConfig config = new WordConfig().setMaxImageWidth(400);
        HtmlParseVo pnVo = WordNodeFactory.html2xml(sampleAnimal.getPreliminaryNotes(), config);
        sampleAnimalObj.put("preliminaryNotes", pnVo.getContent());
        HtmlParseVo dpVo = WordNodeFactory.html2xml(sampleAnimal.getDetectionParameters(), config);
        sampleAnimalObj.put("detectionParameters", dpVo.getContent());
        HtmlParseVo sdpVo = WordNodeFactory.html2xml(sampleAnimal.getSpecialDetectionParameters(), config);
        sampleAnimalObj.put("specialDetectionParameters", sdpVo.getContent());
        images.fluentAddAll(pnVo.getImageCache())
                .fluentAddAll(dpVo.getImageCache())
                .fluentAddAll(sdpVo.getImageCache());

        //申请时间
        sampleInfoObj.put("applyTime", DateUtil.format(sampleInfo.getApplyTime(), DatePattern.NORM_DATE_PATTERN));
        //待提交状态下不显示申请人和申请日期
        if (!GeneStatusEnum.SUBMIT_STATUS.contains(sampleInfo.getStatus())) {
            dataMap.put("createDateStr", DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN));
            String sign = userIdNameCache.getSignBase64(sampleInfo.getApplicantId());
            if (StrUtil.isNotBlank(sign)) {
                dataMap.put("createBySign", sign);
            }
        }

        //查询审核记录
        List<FlowAudit> flowAudits = flowAuditService.auditList(sampleId,"sample." + sampleInfo.getType(), true);
        //项目管理部审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.PROJECT_MANAGEMENT.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("projectManagementDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("projectManagementSign", base64);
                    }
                });
        //部门负责人审核
        flowAudits.stream()
                .filter(audit -> GeneStatusEnum.DEPART_LEADER.getStatus().equals(audit.getFlowStatus()) && StrConstant.YES.equals(audit.getAuditResult()))
                .findFirst()
                .ifPresent(audit -> {
                    dataMap.put("departLeaderDateStr", DateUtil.format(audit.getCreateTime(), DatePattern.CHINESE_DATE_PATTERN));
                    String base64 = wordDataService.getBase64(audit.getCreateSign());
                    if (StrUtil.isNotBlank(base64)) {
                        dataMap.put("departLeaderSign", base64);
                    }
                });

        //项目管理部填写字段
        sampleInfoObj.put("approvalNo", WordUtil.padString(sampleInfo.getApprovalNo(), 14));
        sampleInfoObj.put("revisionNo", WordUtil.padString(sampleInfo.getRevisionNo(), 14));
        sampleInfoObj.put("receiptDate", WordUtil.padString(sampleInfo.getReceiptDate(), 8));
        sampleInfoObj.put("approvalDate", WordUtil.padString(sampleInfo.getApprovalDate(), 8));
        sampleInfoObj.put("expireDate", WordUtil.padString(sampleInfo.getExpireDate(), 8));

        String modelName = "animalSampleTesting.ftl";
        fileInfoService.uploadFileWithFileName(modelName, fileName, sampleInfo.getId(), sampleInfo.getStatus(), dataMap);
    }

    @Override
    public void afterAudit(Long sampleId) {
        SampleEditService.super.afterAudit(sampleId);
        commonSampleEditService.afterAudit(sampleId);
    }

    @Override
    public void afterExecute(Long sampleId) {
        Set<Long> userIds = new HashSet<>();

        //委托单接收人
        List<CommissionVo> commissionList = inspectionCommissionMapper.getCommissionList(sampleId);
        if (CollUtil.isNotEmpty(commissionList)) {
            commissionList.forEach(commission -> {
                if (commission.getSubmitUserId() != null) {
                    userIds.add(commission.getSubmitUserId());
                }
            });
        }
        //送检单接收人、送检人
        List<SampleCheckOrderVo> orderList = checkOrderMapper.getOrderList(sampleId);
        if (CollUtil.isNotEmpty(orderList)) {
            orderList.forEach(order -> {
                if (order.getReceiveUserId() != null) {
                    userIds.add(order.getReceiveUserId());
                }
                if (order.getSendUserId() != null) {
                    userIds.add(order.getSendUserId());
                }
            });
        }

        commonSampleEditService.afterExecuteNotify(sampleId, userIds);
    }

    @Override
    public SampleAnimalVo getDetail(Long sampleId) {
        SampleAnimal sampleAnimal = getOne(new LambdaQueryWrapper<>(SampleAnimal.class)
                .eq(SampleAnimal::getSampleId, sampleId)
                .eq(SampleAnimal::getDeleted, WhetherEnum.NO.getCode()));
        if (sampleAnimal == null) {
            return null;
        }

        SampleAnimalVo sampleAnimalVo = BeanUtil.copyProperties(sampleAnimal, SampleAnimalVo.class);

        //委托列表，列表转为对象，null值转为空对象
        List<CommissionVo> commissionList = inspectionCommissionMapper.getCommissionList(sampleAnimal.getId());
        CommissionObjVo commissionObj = new CommissionObjVo();
        Field[] fields = ReflectUtil.getFields(CommissionObjVo.class);
        for (Field field : fields) {
            String fieldName = field.getName();
            CommissionVo commissionVo = commissionList.stream()
                    .filter(item -> fieldName.equals(item.getDetectType()))
                    .findFirst()
                    .orElse(new CommissionVo());
            ReflectUtil.setFieldValue(commissionObj, fieldName, commissionVo);
        }
        sampleAnimalVo.setCommission(commissionObj);

        //送检申请单，列表转为对象，null值转为空对象
        List<SampleCheckOrderVo> orderList = checkOrderMapper.getOrderList(sampleAnimal.getId());
        CheckOrderObjVo orderObj = new CheckOrderObjVo();
        Field[] orderFields = ReflectUtil.getFields(CheckOrderObjVo.class);
        for (Field field : orderFields) {
            String fieldName = field.getName();
            SampleCheckOrderVo orderVo = orderList.stream()
                    .filter(item -> fieldName.equals(item.getApplyType()))
                    .findFirst()
                    .orElse(new SampleCheckOrderVo());
            ReflectUtil.setFieldValue(orderObj, fieldName, orderVo);
        }
        //血清送检申请单
        Optional<SampleCheckOrderVo> opt = orderList.stream().filter(x -> ScientificSampleTypeEnum.serum.name().equals(x.getApplyType())).findFirst();
        if (opt.isPresent()) {
            SampleCheckOrderVo sampleCheckOrderVo = opt.get();
            SampleCheckSerumVo sampleCheckSerumVo = BeanUtil.copyProperties(sampleCheckOrderVo, SampleCheckSerumVo.class);
            List<SampleCheckItemSerum> serumList = sampleCheckItemSerumService.list(new LambdaQueryWrapper<>(SampleCheckItemSerum.class)
                    .eq(SampleCheckItemSerum::getCheckOrderId, sampleCheckOrderVo.getId()));
            if (CollUtil.isNotEmpty(serumList)) {
                sampleCheckSerumVo.setItems(BeanUtil.copyToList(serumList, CheckItemSerumVo.class));
            }
            orderObj.setSerum(sampleCheckSerumVo);
        }

        sampleAnimalVo.setOrder(orderObj);

        return sampleAnimalVo;
    }
}
